

## The rAUC function calculates the rate-weighted area under the ROC curve (rAUC)
## This version calculates the rAUC in rate-accuracy space
##
## written by Louise AC Millard louise.millard@bristol.ac.uk
##
## Please cite:
## Millard, Louise AC, Peter A. Flach, and Julian PT Higgins. 
## "Rate-constrained ranking and the rate-weighted AUC." 
## Machine Learning and Knowledge Discovery in Databases. Springer Berlin Heidelberg, 2014. 386-403.

# nPlusPrev - number of positives before current section
# nMinusPrev - number of negatives before current section 
# nMinus - The Number of negatives procressed thus far in total
# nPlus - The Number of positives procressed thus far in total
# nTotalMinus - The total number of negative examples in the dataset
# nTotalPlus - The total number of positive examples in the dataset
# pi0 - Class distribution - proportion of positives
# pi1 - Class distribution - proportion of negatives (pi0 + pi1 = 1)
# labelI - label of this current examples, either 0 or 1 (1 is positive !)
# w - a PDF function used to weight across the rates

vauc <- function(nMinusPrev,nPlusPrev, nMinus,nPlus, nTotal, nTotalMinus, nTotalPlus, pi0, pi1, labelI, w) {

    # either f0 or f1 has a constant value for a single move in roc
    # space (no ties) so we use these values or one as a function as
    # the other if for example we need f0 in the equations below but it
    # is f1 that is constant for this step such that f0 is a function
    # of this f0 value and r
    
    f0 = nPlusPrev/nTotalPlus; 
    f1 = nMinusPrev/nTotalMinus;
    
    aucRel = 0;
    
    num1 = nPlusPrev + nMinusPrev; # num examples before this section
    num2 = nPlus + nMinus - 1; # total number processed thus far

	## the accuracy (y-axis in rate-accuracy space) changes with each step along the ranking
	## (i.e. each step right in rate-accuracy space). So we keep track of the positions so we
	## can calculate the accuracy across each rate segment
    for (i in num1:num2) {
        
        if (labelI == 0) {
            nMinusPrev = nMinusPrev + 1;
            f1 = nMinusPrev/nTotalMinus;
            
            f <- function(r) (2*pi0*f0 + pi1 - r - abs(r - pi1)) * w(r);
            fVal = integrate(f,i/nTotal,(i+1)/nTotal);
            aucRel = aucRel + fVal$value;
        }
        else {
            nPlusPrev = nPlusPrev + 1;
            f0 = nPlusPrev/nTotalPlus; 
            f <- function(r) (2*(r - pi1*f1) + pi1 - r - abs(r - pi1))*w(r);
            fVal = integrate(f,i/nTotal,(i+1)/nTotal);
            aucRel = aucRel + fVal$value;
        }
        
    }
    
	return(aucRel);
}


